@@ -37,7 +37,8 @@
 #include "SongEditor/SongEditor.h"
 #include "SongEditor/SongEditorPanel.h"
 #include "PlaylistEditor/PlaylistDialog.h"
-//#include "AudioFileBrowser/AudioFileBrowser.h"
+#include "SampleEditor/SampleEditor.h"
+#include "Director.h"
 
 #include "Mixer/Mixer.h"
 #include "Mixer/MixerLine.h"
@@ -46,7 +47,7 @@
 #include <hydrogen/event_queue.h>
 #include <hydrogen/fx/LadspaFX.h>
 #include <hydrogen/Preferences.h>
-#include <hydrogen/Song.h>
+//#include <hydrogen/sample.h>
 
 #include <QtGui>
 
@@ -65,6 +66,8 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
  , m_pFirstTimeInfo( NULL )
  , m_pPlayerControl( NULL )
  , m_pPlaylistDialog( NULL )
+ , m_pSampleEditor( NULL )
+ , m_pDirector( NULL )
 
 {
 	m_pInstance = this;
@@ -81,11 +84,12 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
 
 	// set initial title
 	QString qsSongName( pFirstSong->__name );
-	if( qsSongName == "Untitled Song" && pFirstSong->get_filename() != "" ){
+	if( qsSongName == "Untitled Song" && !pFirstSong->get_filename().isEmpty() ){
 		qsSongName = pFirstSong->get_filename();
 		qsSongName = qsSongName.section( '/', -1 );
 	}
-	m_pMainForm->setWindowTitle( ( "Hydrogen " + QString( get_version().c_str()) + QString( " - " ) + qsSongName ) );
+
+        setWindowTitle( qsSongName  );
 
 	Preferences *pPref = Preferences::get_instance();
 
@@ -102,7 +106,9 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
 		m_pAudioEngineInfoForm->hide();
 	}
 	
-	 m_pPlaylistDialog = new PlaylistDialog( 0 );
+	m_pPlaylistDialog = new PlaylistDialog( 0 );
+	m_pDirector = new Director( 0 );
+//	m_pSampleEditor = new SampleEditor( 0 );
 	
 	showInfoSplash();	// First time information
 }
@@ -118,6 +124,8 @@ HydrogenApp::~HydrogenApp()
 	delete m_pAudioEngineInfoForm;
 	delete m_pMixer;
 	delete m_pPlaylistDialog;
+	delete m_pDirector;
+	delete m_pSampleEditor;
 
 	Hydrogen *engine = Hydrogen::get_instance();
 	if (engine) {
@@ -132,6 +140,7 @@ HydrogenApp::~HydrogenApp()
 		delete m_pLadspaFXProperties[nFX];
 	}
 	#endif
+	
 }
 
 
@@ -250,6 +259,8 @@ void HydrogenApp::closeFXProperties()
 #endif
 }
 
+
+
 void HydrogenApp::setSong(Song* song)
 {
 
@@ -268,11 +279,11 @@ void HydrogenApp::setSong(Song* song)
 	m_pPatternEditorPanel->updateSLnameLabel();
 
 	QString songName( song->__name );
-	if( songName == "Untitled Song" && song->get_filename() != "" ){
+	if( songName == "Untitled Song" && !song->get_filename().isEmpty() ){
 		songName = song->get_filename();
 		songName = songName.section( '/', -1 );
 	}
-	m_pMainForm->setWindowTitle( ( "Hydrogen " + QString(get_version().c_str()) + QString( " - " ) + songName ) );
+        setWindowTitle( songName  );
 
 	m_pMainForm->updateRecentUsedSongList();
 }
@@ -300,6 +311,9 @@ void HydrogenApp::setStatusBarMessage( const QString& msg, int msec )
 	getPlayerControl()->showMessage( msg, msec );
 }
 
+void HydrogenApp::setWindowTitle( const QString& title){
+    m_pMainForm->setWindowTitle( ( "Hydrogen " + QString( get_version().c_str()) + QString( " - " ) + title ) );
+}
 
 void HydrogenApp::setScrollStatusBarMessage( const QString& msg, int msec, bool test )
 {
@@ -321,6 +335,31 @@ void HydrogenApp::showPlaylistDialog()
 }
 
 
+void HydrogenApp::showDirector()
+{
+	m_pDirector->hide();
+	m_pDirector->show();
+}
+
+
+void HydrogenApp::showSampleEditor( QString name, int mSelectedLayer )
+{
+
+	if ( m_pSampleEditor ){
+		QApplication::setOverrideCursor(Qt::WaitCursor);
+		m_pSampleEditor->close();
+		delete m_pSampleEditor;
+		m_pSampleEditor = NULL;
+		QApplication::restoreOverrideCursor();
+	}
+	QApplication::setOverrideCursor(Qt::WaitCursor);	
+	m_pSampleEditor = new SampleEditor( 0, mSelectedLayer, name);
+	m_pSampleEditor->show();
+	QApplication::restoreOverrideCursor();
+}
+
+
+
 void HydrogenApp::showInfoSplash()
 {
 	QString sDocPath( DataPath::get_data_path().append( "/doc/infoSplash" ) );
@@ -331,7 +370,7 @@ void HydrogenApp::showInfoSplash()
 		return;
 	}
 
-	QString sFilename = "";
+	QString sFilename;
 	int nNewsID = 0;
 	QFileInfoList list = dir.entryInfoList();
 
@@ -353,7 +392,7 @@ void HydrogenApp::showInfoSplash()
 	INFOLOG( "[showInfoSplash] Selected news: " + sFilename );
 
 	QString sLastRead = Preferences::get_instance()->getLastNews();
-	if ( sLastRead != sFilename && sFilename != "" ) {
+	if ( sLastRead != sFilename && !sFilename.isEmpty() ) {
 		QString sDocURI = sDocPath;
 		sDocURI.append( "/" ).append( sFilename );
 		SimpleHTMLBrowser *m_pFirstTimeInfo = new SimpleHTMLBrowser( m_pMainForm, sDocPath, sDocURI, SimpleHTMLBrowser::WELCOME );
@@ -368,6 +407,11 @@ void HydrogenApp::onDrumkitLoad( QString name ){
 	m_pPatternEditorPanel->updateSLnameLabel( );
 }
 
+void HydrogenApp::enableDestructiveRecMode(){
+	m_pPatternEditorPanel->displayorHidePrePostCB();
+}
+
+
 void HydrogenApp::onEventQueueTimer()
 {
 	// use the timer to do schedule instrument slaughter;
@@ -419,6 +463,10 @@ void HydrogenApp::onEventQueueTimer()
 					pListener->metronomeEvent( event.value );
 					break;
 
+				case EVENT_RECALCULATERUBBERBAND:
+					pListener->rubberbandbpmchangeEvent();
+					break;
+
 				case EVENT_PROGRESS:
 					pListener->progressEvent( event.value );
 					break;
