@@ -21,6 +21,7 @@
  */
 #include "version.h"
 
+
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/playlist.h>
 #include <hydrogen/audio_engine.h>
@@ -43,6 +44,7 @@ using namespace H2Core;
 #include "LadspaFXProperties.h"
 #include "SongPropertiesDialog.h"
 
+#include "Director.h"
 #include "Mixer/Mixer.h"
 #include "InstrumentEditor/InstrumentEditorPanel.h"
 #include "PatternEditor/PatternEditorPanel.h"
@@ -52,11 +54,13 @@ using namespace H2Core;
 #include "SoundLibrary/SoundLibraryImportDialog.h"
 #include "SoundLibrary/SoundLibrarySaveDialog.h"
 #include "SoundLibrary/SoundLibraryExportDialog.h"
+#include "PlaylistEditor/PlaylistDialog.h"
 
 #include <QtGui>
 
 #ifndef WIN32
 	#include <sys/time.h>
+        #include <sys/socket.h>
 #endif
 
 #ifdef LASH_SUPPORT
@@ -70,6 +74,8 @@ using namespace H2Core;
 using namespace std;
 using namespace H2Core;
 
+int MainForm::sigusr1Fd[2];
+
 MainForm::MainForm( QApplication *app, const QString& songFilename )
  : QMainWindow( 0, 0 )
  , Object( "MainForm" )
@@ -77,13 +83,21 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 	setMinimumSize( QSize( 1000, 600 ) );
 	setWindowIcon( QPixmap( Skin::getImagePath() + "/icon16.png" ) );
 
+    #ifndef WIN32
+	if (::socketpair(AF_UNIX, SOCK_STREAM, 0, sigusr1Fd))
+	   qFatal("Couldn't create HUP socketpair");
+	snUsr1 = new QSocketNotifier(sigusr1Fd[1], QSocketNotifier::Read, this);
+	connect(snUsr1, SIGNAL(activated(int)), this, SLOT( handleSigUsr1() ));
+    #endif
+
+
 	m_pQApp = app;
 
 	m_pQApp->processEvents();
 
 	// Load default song
 	Song *song = NULL;
-	if (songFilename != "") {
+	if ( !songFilename.isEmpty() ) {
 		song = Song::load( songFilename );
 		if (song == NULL) {
 			//QMessageBox::warning( this, "Hydrogen", trUtf8("Error loading song.") );
@@ -95,7 +109,7 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 		Preferences *pref = Preferences::get_instance();
 		bool restoreLastSong = pref->isRestoreLastSongEnabled();
 		QString filename = pref->getLastSongFilename();
-		if ( restoreLastSong && (filename != "" )) {
+		if ( restoreLastSong && ( !filename.isEmpty() )) {
 			song = Song::load( filename );
 			if (song == NULL) {
 				//QMessageBox::warning( this, "Hydrogen", trUtf8("Error restoring last song.") );
@@ -122,16 +136,16 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 	// for all the window modes
 	h2app->getMixer()->installEventFilter (this);
 	h2app->getPatternEditorPanel()->installEventFilter (this);
+	h2app->getPatternEditorPanel()->getPianoRollEditor()->installEventFilter (this);
 	h2app->getSongEditorPanel()->installEventFilter (this);
 	h2app->getPlayerControl()->installEventFilter(this);
 	InstrumentEditorPanel::get_instance()->installEventFilter(this);
 	h2app->getAudioEngineInfoForm()->installEventFilter(this);
-
+	h2app->getDirector()->installEventFilter(this);
+//	h2app->getPlayListDialog()->installEventFilter(this);
 	installEventFilter( this );
 
-	connect(&m_http, SIGNAL(done(bool)), this, SLOT(latestVersionDone(bool)));
-	getLatestVersion();
-
+	showDevelWarning();
 
 	connect( &m_autosaveTimer, SIGNAL(timeout()), this, SLOT(onAutoSaveTimer()));
 	m_autosaveTimer.start( 60 * 1000 );
@@ -167,7 +181,17 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 	
 //beatcouter
 	Hydrogen::get_instance()->setBcOffsetAdjust();
+// director
+	EventQueue::get_instance()->push_event( EVENT_METRONOME, 1 );
+	EventQueue::get_instance()->push_event( EVENT_METRONOME, 3 );
 
+	//restore last playlist
+	if( Preferences::get_instance()->isRestoreLastPlaylistEnabled() ){
+		bool loadlist = h2app->getPlayListDialog()->loadListByFileName( Preferences::get_instance()->getLastPlaylistFilename() );
+		if( !loadlist ){
+			_ERRORLOG ( "Error loading the playlist" );
+		}
+	}
 }
 
 
@@ -176,7 +200,10 @@ MainForm::~MainForm()
 {
 	// remove the autosave file
 	QFile file( getAutoSaveFilename() );
-	file.remove();
+        file.remove();
+	
+	//if a playlist is used, we save the last playlist-path to hydrogen.conf
+	Preferences::get_instance()->setLastPlaylistFilename( Playlist::get_instance()->__playlistName );
 
 	if ( (Hydrogen::get_instance()->getState() == STATE_PLAYING) ) {
 		Hydrogen::get_instance()->sequencer_stop();
@@ -266,6 +293,7 @@ void MainForm::createMenuBar()
 //		m_pWindowMenu->addAction( trUtf8("Show song editor"), this, SLOT( action_window_showSongEditor() ), QKeySequence( "" ) );
 //	}
 	m_pToolsMenu->addAction( trUtf8("Playlist &editor"), this, SLOT( action_window_showPlaylistDialog() ), QKeySequence( "" ) );
+	m_pToolsMenu->addAction( trUtf8("Director"), this, SLOT( action_window_show_DirectorWidget() ), QKeySequence( "Alt+D" ) );
 
 	m_pToolsMenu->addAction( trUtf8("&Mixer"), this, SLOT( action_window_showMixer() ), QKeySequence( "Alt+M" ) );
 
@@ -310,8 +338,8 @@ if ( Preferences::get_instance()->useLash() ){
 
 	lash_event_t* event;
 
-	string songFilename = "";
-	QString filenameSong = "";
+	string songFilename;
+	QString filenameSong;
 	Song *song = Hydrogen::get_instance()->getSong();
 	// Extra parentheses for -Wparentheses
 	while ( (event = client->getNextEvent()) ) {
@@ -325,7 +353,7 @@ if ( Preferences::get_instance()->useLash() ){
 				songFilename.append(lash_event_get_string(event));
 				songFilename.append("/hydrogen.h2song"); 
 				
-				filenameSong = songFilename.c_str();
+				filenameSong = QString::fromLocal8Bit( songFilename.c_str() );
 				song->set_filename( filenameSong );
 				action_file_save();
 			  
@@ -341,7 +369,7 @@ if ( Preferences::get_instance()->useLash() ){
 				INFOLOG( QString("[LASH] Restore file: %1")
 					 .arg( songFilename.c_str() ) );
 
-				filenameSong = songFilename.c_str();
+				filenameSong = QString::fromLocal8Bit( songFilename.c_str() );
 							 
 				openSongFile( filenameSong );
 			 
@@ -400,11 +428,16 @@ void MainForm::action_file_new()
 		return;
 	}
 
+	Hydrogen::get_instance()->m_timelinevector.clear();
 	Song * song = Song::get_empty_song();
 	song->set_filename( "" );
 	h2app->setSong(song);
  	Hydrogen::get_instance()->setSelectedPatternNumber( 0 );
 	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->update_background_color();
+	HydrogenApp::get_instance()->getSongEditorPanel()->updatePositionRuler();
+	Hydrogen::get_instance()->m_timelinetagvector.clear();
+	EventQueue::get_instance()->push_event( EVENT_METRONOME, 2 );
+	EventQueue::get_instance()->push_event( EVENT_METRONOME, 3 );
 }
 
 
@@ -418,14 +451,14 @@ void MainForm::action_file_save_as()
         std::auto_ptr<QFileDialog> fd( new QFileDialog );
 	fd->setFileMode( QFileDialog::AnyFile );
 	fd->setFilter( trUtf8("Hydrogen Song (*.h2song)") );
-	fd->setAcceptMode( QFileDialog::AcceptSave );
-	fd->setWindowTitle( trUtf8( "Save song" ) );
+        fd->setAcceptMode( QFileDialog::AcceptSave );
+        fd->setWindowTitle( trUtf8( "Save song" ) );
 
 	Song *song = Hydrogen::get_instance()->getSong();
 	QString defaultFilename;
 	QString lastFilename = song->get_filename();
 
-	if (lastFilename == "") {
+	if ( lastFilename.isEmpty() ) {
 		defaultFilename = Hydrogen::get_instance()->getSong()->__name;
 		defaultFilename += ".h2song";
 	}
@@ -435,12 +468,12 @@ void MainForm::action_file_save_as()
 
 	fd->selectFile( defaultFilename );
 
-	QString filename = "";
+	QString filename;
 	if (fd->exec() == QDialog::Accepted) {
 		filename = fd->selectedFiles().first();
 	}
 
-	if (filename != "") {
+	if ( !filename.isEmpty() ) {
 		QString sNewFilename = filename;
 		if ( sNewFilename.endsWith(".h2song") == false ) {
 			filename += ".h2song";
@@ -450,9 +483,7 @@ void MainForm::action_file_save_as()
 		action_file_save();
 	}
 	h2app->setScrollStatusBarMessage( trUtf8("Song saved as.") + QString(" Into: ") + defaultFilename, 2000 );
-	//update SoundlibraryPanel
-	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->test_expandedItems();
-	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->updateDrumkitList();
+        h2app->setWindowTitle( filename );
 }
 
 
@@ -466,7 +497,7 @@ void MainForm::action_file_save()
 	Song *song = Hydrogen::get_instance()->getSong();
 	QString filename = song->get_filename();
 
-	if (filename == "") {
+	if ( filename.isEmpty() ) {
 		// just in case!
 		return action_file_save_as();
 	}
@@ -491,9 +522,6 @@ void MainForm::action_file_save()
 
 		h2app->setScrollStatusBarMessage( trUtf8("Song saved.") + QString(" Into: ") + filename, 2000 );
 	}
-	//update SoundlibraryPanel
-	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->test_expandedItems();
-	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->updateDrumkitList();
 }
 
 
@@ -542,7 +570,7 @@ void MainForm::action_file_export_pattern_as()
 	fd->setFileMode ( QFileDialog::AnyFile );
 	fd->setFilter ( trUtf8 ( "Hydrogen Pattern (*.h2pattern)" ) );
 	fd->setAcceptMode ( QFileDialog::AcceptSave );
-	fd->setWindowTitle ( trUtf8 ( "Save Pattern as ..." ) );
+        fd->setWindowTitle ( trUtf8 ( "Save Pattern as ..." ) );
 	fd->setDirectory ( dir );
 
 
@@ -552,7 +580,7 @@ void MainForm::action_file_export_pattern_as()
 	fd->selectFile ( defaultPatternname );
 
 	LocalFileMng fileMng;
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted )
 	{
 		filename = fd->selectedFiles().first();
@@ -562,7 +590,7 @@ void MainForm::action_file_export_pattern_as()
 		Preferences::get_instance()->__lastspatternDirectory = newdatapath;
 	}
 
-	if ( filename != "" )
+	if ( !filename.isEmpty() )
 	{
 		QString sNewFilename = filename;
 		if(sNewFilename.endsWith( ".h2pattern" ) ){
@@ -583,6 +611,7 @@ void MainForm::action_file_export_pattern_as()
 		{
 			QMessageBox::warning( this, "Hydrogen", trUtf8("Could not export pattern.") );
 			_ERRORLOG ( "Error saving the pattern" );
+			QMessageBox::warning( this, "Hydrogen", trUtf8("Could not export pattern.") ); 
 		}
 	}
 	h2app->setStatusBarMessage ( trUtf8 ( "Pattern saved." ), 10000 );
@@ -611,7 +640,7 @@ void MainForm::action_file_open() {
 	fd->setFilter( trUtf8("Hydrogen Song (*.h2song)") );
 	fd->setDirectory( lastUsedDir );
 
-	fd->setWindowTitle( trUtf8( "Open song" ) );
+        fd->setWindowTitle( trUtf8( "Open song" ) );
 
 	/// \todo impostare il preview
 	/*
@@ -620,14 +649,14 @@ void MainForm::action_file_open() {
 	fd->setPreviewMode( QFileDialog::Contents );
 	*/
 
-	QString filename = "";
+	QString filename;
 	if (fd->exec() == QDialog::Accepted) {
 		filename = fd->selectedFiles().first();
 		lastUsedDir = fd->directory().absolutePath();
 	}
 
 
-	if (filename != "") {
+	if ( !filename.isEmpty() ) {
 		openSongFile( filename );
 	}
 
@@ -654,7 +683,7 @@ void MainForm::action_file_openPattern()
 	fd->setWindowTitle ( trUtf8 ( "Open Pattern" ) );
 
 
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted )
 	{
 		filename = fd->selectedFiles().first();
@@ -708,13 +737,13 @@ void MainForm::action_file_openDemo()
 	fd->setDirectory( QString( Preferences::get_instance()->getDemoPath() ) );
 
 
-	QString filename = "";
+	QString filename;
 	if (fd->exec() == QDialog::Accepted) {
 		filename = fd->selectedFiles().first();
 	}
 
 
-	if (filename != "") {
+	if ( !filename.isEmpty() ) {
 		openSongFile( filename );
 		Hydrogen::get_instance()->getSong()->set_filename( "" );
 	}
@@ -738,7 +767,11 @@ void MainForm::action_window_showPlaylistDialog()
 	h2app->showPlaylistDialog();	
 }
 
+void MainForm::action_window_show_DirectorWidget()
+{
 
+	h2app->showDirector();
+}
 
 void MainForm::action_window_showMixer()
 {
@@ -781,7 +814,7 @@ void MainForm::action_instruments_addInstrument()
 	}
 	++nID;
 
-	Instrument *pNewInstr = new Instrument(QString( nID ), "New instrument", new ADSR());
+	Instrument *pNewInstr = new Instrument( QString::number( nID ), "New instrument", new ADSR());
 	pList->add( pNewInstr );
 	
 	#ifdef JACK_SUPPORT
@@ -1055,12 +1088,12 @@ void MainForm::updateRecentUsedSongList()
 	Preferences *pPref = Preferences::get_instance();
 	vector<QString> recentUsedSongs = pPref->getRecentFiles();
 
-	QString sFilename = "";
+	QString sFilename;
 
 	for ( uint i = 0; i < recentUsedSongs.size(); ++i ) {
 		sFilename = recentUsedSongs[ i ];
 
-		if ( sFilename != "" ) {
+		if ( !sFilename.isEmpty() ) {
 			QAction *pAction = new QAction( this  );
 			pAction->setText( sFilename );
 			m_pRecentFilesMenu->addAction( pAction );
@@ -1085,6 +1118,8 @@ void MainForm::openSongFile( const QString& sFilename )
                 engine->sequencer_stop();
 	}
 
+	engine->m_timelinetagvector.clear();
+
 	h2app->closeFXProperties();
 	LocalFileMng mng;
 	Song *pSong = Song::load( sFilename );
@@ -1093,6 +1128,7 @@ void MainForm::openSongFile( const QString& sFilename )
 		return;
 	}
 
+
 	// add the new loaded song in the "last used song" vector
 	Preferences *pPref = Preferences::get_instance();
 	vector<QString> recentFiles = pPref->getRecentFiles();
@@ -1103,6 +1139,9 @@ void MainForm::openSongFile( const QString& sFilename )
 
 	updateRecentUsedSongList();
 	engine->setSelectedPatternNumber( 0 );
+	HydrogenApp::get_instance()->getSongEditorPanel()->updatePositionRuler();
+//	EventQueue::get_instance()->push_event( EVENT_METRONOME, 1 );
+	EventQueue::get_instance()->push_event( EVENT_METRONOME, 3 );
 }
 
 
@@ -1353,7 +1392,7 @@ bool MainForm::eventFilter( QObject *o, QEvent *e )
 			float pan_L = 1.0;
 			float pan_R = 1.0;
 
-			engine->addRealtimeNote (row, velocity, pan_L, pan_R);
+			engine->addRealtimeNote (row, velocity, pan_L, pan_R, NULL, NULL, NULL , row + 36);
 
 			return TRUE; // eat event
 		}
@@ -1396,12 +1435,12 @@ void MainForm::action_file_export_midi()
 	fd->setAcceptMode( QFileDialog::AcceptSave );
 //	fd->setIcon( QPixmap( Skin::getImagePath() + "/icon16.png" ) );
 
-	QString sFilename = "";
+	QString sFilename;
 	if ( fd->exec() == QDialog::Accepted ) {
 		sFilename = fd->selectedFiles().first();
 	}
 
-	if ( sFilename != "" ) {
+	if ( !sFilename.isEmpty() ) {
 		if ( sFilename.endsWith(".mid") == false ) {
 			sFilename += ".mid";
 		}
@@ -1472,115 +1511,29 @@ void MainForm::action_window_showPatternEditor()
 }
 
 
-///
-/// Retrieve from the website the latest version available.
-///
-/// Warning: Hydrogen is not a spyware!!
-/// Hydrogen sends only the current version and the OS used in order to let possible
-/// to use an auto-updater in the future (this feature is not ready yet).
-///
-/// *** No user data will be stored in the server ***
-///
-void MainForm::getLatestVersion()
+void MainForm::showDevelWarning()
 {
-	#if defined( Q_OS_MACX )
-	QString os = "Mac";
-	#elif defined( Q_OS_WIN32 )
-	QString os = "Windows";
-	#elif defined( Q_OS_WIN64 )
-	QString os = "Windows64";
-	#elif defined( Q_OS_LINUX )
-	QString os = "Linux";
-	#elif defined( Q_OS_FREEBSD )
-	QString os = "FreeBSD";
-	#elif defined( Q_OS_UNIX )
-	QString os = "Unix";
-	#else
-	QString os = "Unknown";
-	#endif
+
+	//set this to 'false' for the case that you want to make a release..
+	if ( true ) {
+                Preferences *pref = Preferences::get_instance();
+                bool isDevelWarningEnabled = pref->getShowDevelWarning();
+                if(isDevelWarningEnabled) {
+
+                        QString msg = trUtf8( "You're using a development version of Hydrogen, please help us reporting bugs or suggestions in the hydrogen-devel mailing list.<br><br>Thank you!" );
+                        QMessageBox develMessageBox( this );
+                        develMessageBox.setText( msg );
+                        develMessageBox.addButton( QMessageBox::Ok );
+                        develMessageBox.addButton( trUtf8( "Don't show this message anymore" ) , QMessageBox::AcceptRole );
+
+                        if( develMessageBox.exec() == 0 ){
+                                //don't show warning again
+                                pref->setShowDevelWarning( false );
+                        }
+          }
 
 
-	QString sRequest = QString("/getLatestVersion.php?UsingVersion=%1").arg( get_version().c_str() );
-	sRequest += QString( "&OS=%1" ).arg( os );
-
-	//INFOLOG( sRequest );
-
-	QHttpRequestHeader header( "GET", sRequest );
-	header.setValue( "Host", "www.hydrogen-music.org" );
-
-	m_http.setHost( "www.hydrogen-music.org" );
-	m_http.request( header );
-}
-
-
-
-void MainForm::latestVersionDone(bool bError)
-{
-	if ( bError ) {
-		INFOLOG( "[MainForm::latestVersionDone] Error." );
-		return;
-	}
-
-	QString sLatestVersion( m_http.readAll() );
-	sLatestVersion = sLatestVersion.simplified();
-	QString sLatest_major = sLatestVersion.section( '.', 0, 0 );
-	QString sLatest_minor = sLatestVersion.section( '.', 1, 1 );
-	QString sLatest_micro = sLatestVersion.section( '.', 2, 2 );
-//	INFOLOG( "Latest available version is: " + QString( sLatestVersion.ascii() ) );
-
-	QString sCurrentVersion = get_version().c_str();
-	QString sCurrent_major = sCurrentVersion.section( '.', 0, 0 );
-	QString sCurrent_minor = sCurrentVersion.section( '.', 1, 1 );
-	QString sCurrent_micro = sCurrentVersion.section( '.', 2, 2 );
-	if ( sCurrent_micro.section( '-', 0, 0 ) != "" ) {
-		sCurrent_micro = sCurrent_micro.section( '-', 0, 0 );
-	}
-
-	bool bExistsNewVersion = false;
-	if ( sLatest_major > sCurrent_major ) {
-		bExistsNewVersion = true;
-	}
-	else if ( sLatest_minor > sCurrent_minor ) {
-			bExistsNewVersion = true;
-	}
-	else if ( sLatest_micro > sCurrent_micro ) {
-		bExistsNewVersion = true;
-	}
-
-	bool bUsingDevelVersion = false;
-	if ( sLatest_major < sCurrent_major ) {
-		bUsingDevelVersion = true;
-	}
-	else if ( sLatest_major == sCurrent_major && sLatest_minor < sCurrent_minor ) {
-		bUsingDevelVersion = true;
-	}
-	else if ( sLatest_major == sCurrent_major && sLatest_minor == sCurrent_minor && sLatest_micro < sCurrent_micro ) {
-		bUsingDevelVersion = true;
-	}
-
-	if ( bExistsNewVersion ) {
-		QString sLatest = QString(sLatest_major) + "." +  QString(sLatest_minor) + "." + QString(sLatest_micro);
-		WARNINGLOG( "\n\n*** A newer version (v" + sLatest + ") of Hydrogen is available at http://www.hydrogen-music.org\n" );
-	}
-//	if ( bUsingDevelVersion ) {
-//		Preferences *pref = Preferences::get_instance();
-//		bool isDevelWarningEnabled = pref->getShowDevelWarning();
-//		if(isDevelWarningEnabled) {
-//
-//			QString msg = trUtf8( "You're using a development version of Hydrogen, please help us reporting bugs or suggestions in the hydrogen-devel mailing list.<br><br>Thank you!" );
-//			QMessageBox develMessageBox( this );
-//			develMessageBox.setText( msg );
-//			develMessageBox.addButton( QMessageBox::Ok );
-//			develMessageBox.addButton( trUtf8( "Don't show this message anymore" ) , QMessageBox::AcceptRole );
-//
-//			if( develMessageBox.exec() == 0 ){
-//				//don't show warning again
-//				pref->setShowDevelWarning( false );
-//			}
-//	  }
-//
-//
-//	}
+        }
 }
 
 
@@ -1592,7 +1545,7 @@ QString MainForm::getAutoSaveFilename()
 	QString sOldFilename = pSong->get_filename();
 	QString newName = "autosave.h2song";
 
-	if ( sOldFilename != "" ) {
+	if ( !sOldFilename.isEmpty() ) {
 		newName = sOldFilename.left( sOldFilename.length() - 7 ) + ".autosave.h2song";
 	}
 
@@ -1629,7 +1582,7 @@ void MainForm::onPlaylistDisplayTimer()
 	if( Hydrogen::get_instance()->m_PlayList.size() == 0)
 		return;
 	int songnumber = Playlist::get_instance()->getActiveSongNumber();
-	QString songname = "";
+	QString songname;
 	if ( songnumber == -1 )
 			return;
 
@@ -1659,7 +1612,7 @@ bool MainForm::handleUnsavedChanges()
 						2 ) ) { // Escape == button 2
 			case 0: // Save clicked or Alt+S pressed or Enter pressed.
 				// If the save fails, the __is_modified flag will still be true
-				if ( Hydrogen::get_instance()->getSong()->get_filename() != "") {
+			    if ( ! Hydrogen::get_instance()->getSong()->get_filename().isEmpty() ) {
 					action_file_save();
 				} else {
 					// never been saved
@@ -1680,3 +1633,21 @@ bool MainForm::handleUnsavedChanges()
 	}
 	return rv;
 }
+
+void MainForm::usr1SignalHandler(int)
+ {
+     char a = 1;
+     ::write(sigusr1Fd[0], &a, sizeof(a));
+ }
+
+void MainForm::handleSigUsr1()
+{
+    snUsr1->setEnabled(false);
+    char tmp;
+    ::read(sigusr1Fd[1], &tmp, sizeof(tmp));
+
+    action_file_save();
+    snUsr1->setEnabled(true);
+}
+
+
