@@ -25,6 +25,7 @@
 #include "../HydrogenApp.h"
 #include "../InstrumentRack.h"
 #include "SoundLibrary/SoundLibraryPanel.h"
+#include "SongEditor/SongEditorPanel.h"
 #include "widgets/PixmapWidget.h"
 
 #include <hydrogen/LocalFileMng.h>
@@ -32,6 +33,7 @@
 #include <hydrogen/Preferences.h>
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/playlist.h>
+#include <hydrogen/event_queue.h>
 
 #include "../widgets/Button.h"
 
@@ -56,10 +58,10 @@ PlaylistDialog::PlaylistDialog ( QWidget* pParent )
 
 	setupUi ( this );
 	INFOLOG ( "INIT" );
-	setWindowTitle ( trUtf8 ( "Play List Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
+	setWindowTitle ( trUtf8 ( "Playlist Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
 	setFixedSize ( width(), height() );
-	installEventFilter(this);
 
+	installEventFilter( this );
 
 	// menubar
 	QMenuBar *m_pMenubar = new QMenuBar( this );
@@ -72,7 +74,7 @@ PlaylistDialog::PlaylistDialog ( QWidget* pParent )
 	m_pPlaylistMenu->addAction( trUtf8( "Add &current song to Playlist" ), this, SLOT( addCurrentSong() ), QKeySequence( "" ) );
 	m_pPlaylistMenu->addSeparator();				// -----
 	m_pPlaylistMenu->addAction( trUtf8( "&Remove selected song from Playlist" ), this, SLOT( removeFromList() ), QKeySequence( "" ) );
-	m_pPlaylistMenu->addAction( trUtf8( "Remove all songs from &Playlistist " ), this, SLOT( clearPlaylist() ), QKeySequence( "" ) );
+	m_pPlaylistMenu->addAction( trUtf8( "Remove all songs from &Playlist" ), this, SLOT( clearPlaylist() ), QKeySequence( "" ) );
 	m_pPlaylistMenu->addSeparator();
 	m_pPlaylistMenu->addAction( trUtf8( "&Open Playlist" ), this, SLOT( loadList() ), QKeySequence( "" ) );
 	m_pPlaylistMenu->addSeparator();
@@ -80,7 +82,7 @@ PlaylistDialog::PlaylistDialog ( QWidget* pParent )
 	m_pPlaylistMenu->addAction( trUtf8( "Save Playlist &as" ), this, SLOT( saveListAs() ), QKeySequence( "" ) );
 
 #ifdef WIN32
-	//no scripts under windows
+        //no scripts under windows
 #else
 	// Script menu
 	QMenu *m_pScriptMenu = m_pMenubar->addMenu( trUtf8( "&Scripts" ) );
@@ -160,7 +162,7 @@ PlaylistDialog::PlaylistDialog ( QWidget* pParent )
 	m_pPlaylistTree->setHeaderItem ( header );
 	m_pPlaylistTree->setAlternatingRowColors( true );
 
-	addSongBTN->setEnabled ( true );
+        /*addSongBTN->setEnabled ( true );
 	loadListBTN->setEnabled ( true );
 	removeFromListBTN->setEnabled ( false );
 	removeFromListBTN->setEnabled ( false );
@@ -170,7 +172,7 @@ PlaylistDialog::PlaylistDialog ( QWidget* pParent )
 	removeScriptBTN->hide();
 	editScriptBTN->hide();
 	newScriptBTN->hide();
-	clearPlBTN->setEnabled ( false );
+        clearPlBTN->setEnabled ( false );*/
 
 	QVBoxLayout *sideBarLayout = new QVBoxLayout(sideBarWidget);
 	sideBarLayout->setSpacing(0);
@@ -272,16 +274,19 @@ void PlaylistDialog::addSong()
 	static QString songDir = Preferences::get_instance()->getDataDirectory()  + "/songs";;
 
 	std::auto_ptr<QFileDialog> fd( new QFileDialog );
-	fd->setFileMode ( QFileDialog::ExistingFile );
+	fd->setFileMode ( QFileDialog::ExistingFiles );
 	fd->setFilter ( "Hydrogen song (*.h2song)" );
 	fd->setDirectory ( songDir );
 
 	fd->setWindowTitle ( trUtf8 ( "Add Song to PlayList" ) );
 
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted ){
-		filename = fd->selectedFiles().first();
-		updatePlayListNode ( filename );
+		int i;
+		for(i=0; i < fd->selectedFiles().size(); i++){ 
+			filename = fd->selectedFiles().at(i);
+			updatePlayListNode ( filename );
+		}
 	}
 
 
@@ -324,7 +329,7 @@ void PlaylistDialog::removeFromList()
 			Playlist::get_instance()->setSelectedSongNr( -1 );
 			Playlist::get_instance()->setActiveSongNumber( -1 );
 			Playlist::get_instance()->__playlistName = "";
-			setWindowTitle ( trUtf8 ( "Play List Browser" ) );
+			setWindowTitle ( trUtf8 ( "Playlist Browser" ) );
 			return;
 		}else
 		{	
@@ -351,7 +356,7 @@ void PlaylistDialog::clearPlaylist()
 	Playlist::get_instance()->setSelectedSongNr( -1 );
 	Playlist::get_instance()->setActiveSongNumber( -1 );
 	Playlist::get_instance()->__playlistName = "";
-	setWindowTitle ( trUtf8 ( "Play List Browser" ) );
+	setWindowTitle ( trUtf8 ( "Playlist Browser" ) );
 	return;	
 }
 
@@ -361,7 +366,7 @@ void PlaylistDialog::updatePlayListNode ( QString file )
 
 	QTreeWidgetItem* m_pPlaylistItem = new QTreeWidgetItem ( m_pPlaylistTree );
 	m_pPlaylistItem->setText ( 0, file );
-	m_pPlaylistItem->setText ( 1, "no Script" );
+        m_pPlaylistItem->setText ( 1, trUtf8 ("no Script") );
 	m_pPlaylistItem->setCheckState( 2, Qt::Unchecked );
 
 	updatePlayListVector();
@@ -383,16 +388,15 @@ void PlaylistDialog::loadList()
 
 	fd->setWindowTitle ( trUtf8 ( "Load Playlist" ) );
 
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted ){
 		filename = fd->selectedFiles().first();
 
 		LocalFileMng fileMng;
-		int err = fileMng.loadPlayList( filename.toStdString() );
+		int err = fileMng.loadPlayList( filename.toLocal8Bit().constData() );
 		if ( err != 0 ) {
-			_ERRORLOG( "Error saving the playlist" );
+			_ERRORLOG( "Error loading the playlist" );
 		}
-
 	
 		if(Hydrogen::get_instance()->m_PlayList.size() > 0){
 
@@ -414,7 +418,7 @@ void PlaylistDialog::loadList()
 			m_pPlaylist->setCurrentItem ( m_pPlaylistItem );
 			Playlist::get_instance()->setSelectedSongNr( 0 );
 			Playlist::get_instance()->__playlistName = filename;
-			setWindowTitle ( trUtf8 ( "Play List Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
+			setWindowTitle ( trUtf8 ( "Playlist Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
 		}
 
 	}
@@ -441,7 +445,7 @@ void PlaylistDialog::newScript()
 
 	fd->selectFile ( defaultFilename );
 
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted )
 	{
 		filename = fd->selectedFiles().first();
@@ -470,7 +474,7 @@ void PlaylistDialog::newScript()
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "WARNING, the new file is executable by the owner of the file!" ) );
 	}
 	
-	if( pPref->getDefaultEditor() == ""){
+	if( pPref->getDefaultEditor().isEmpty() ){
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Default Editor Set. Please set your Default Editor\nDo not use a console based Editor\nSorry, but this will not work for the moment." ) );
 
 		static QString lastUsedDir = "/usr/bin/";
@@ -481,7 +485,7 @@ void PlaylistDialog::newScript()
 	
 		fd->setWindowTitle ( trUtf8 ( "Set your Default Editor" ) );
 	
-		QString filename = "";
+		QString filename;
 		if ( fd->exec() == QDialog::Accepted ){
 			filename = fd->selectedFiles().first();
 	
@@ -521,7 +525,7 @@ void PlaylistDialog::saveListAs()
 
 	fd->selectFile ( defaultFilename );
 
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted )
 	{
 		filename = fd->selectedFiles().first();
@@ -531,13 +535,13 @@ void PlaylistDialog::saveListAs()
 	}
 
 	LocalFileMng fileMng;
-	int err = fileMng.savePlayList( filename.toStdString() );
+	int err = fileMng.savePlayList( filename.toLocal8Bit().constData() );
 	if ( err != 0 ) {
 		_ERRORLOG( "Error saving the playlist" );
 	}else
 	{
 		Playlist::get_instance()->__playlistName = filename;
-		setWindowTitle ( trUtf8 ( "Play List Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
+		setWindowTitle ( trUtf8 ( "Playlist Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
 	}
 }
 
@@ -576,7 +580,7 @@ void PlaylistDialog::loadScript()
 	fd->setFilter ( trUtf8 ( "Hydrogen Playlist (*.sh)" ) );
 	fd->setWindowTitle ( trUtf8 ( "Add Script to selected Song" ) );
 
-	QString filename = "";
+	QString filename;
 	if ( fd->exec() == QDialog::Accepted ){
 		filename = fd->selectedFiles().first();
 //		filename = filename.simplified();
@@ -602,14 +606,14 @@ void PlaylistDialog::removeScript()
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Song selected!" ));
 		return;
 	}else{
-		QString selected = "";
+		QString selected;
 		selected = m_pPlaylistItem->text ( 1 );
-		if( selected == "no Script"){
+                if( !QFile( selected ).exists() ){
 			QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Script in use!" ));
 			return;
 		}else
 		{
-			m_pPlaylistItem->setText ( 1, "no Script" );
+                        m_pPlaylistItem->setText ( 1, trUtf8 ("no Script") );
 			m_pPlaylistItem->setCheckState( 2, Qt::Unchecked );
 			updatePlayListVector();
 		}
@@ -621,7 +625,7 @@ void PlaylistDialog::removeScript()
 void PlaylistDialog::editScript()
 {
 	Preferences *pPref = Preferences::get_instance();
-	if( pPref->getDefaultEditor() == ""){
+	if( pPref->getDefaultEditor().isEmpty() ){
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Default Editor Set. Please set your Default Editor\nDo not use a console based Editor\nSorry, but this will not work for the moment." ) );
 
 		static QString lastUsedDir = "/usr/bin/";
@@ -632,7 +636,7 @@ void PlaylistDialog::editScript()
 	
 		fd->setWindowTitle ( trUtf8 ( "Set your Default Editor" ) );
 	
-		QString filename = "";
+		QString filename;
 		if ( fd->exec() == QDialog::Accepted ){
 			filename = fd->selectedFiles().first();
 	
@@ -646,12 +650,12 @@ void PlaylistDialog::editScript()
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Song selected!" ) );
 		return;
 	}
-	QString selected = "";
+	QString selected;
 	selected = m_pPlaylistItem->text ( 1 );
 
 	QString filename = pPref->getDefaultEditor() + " " + selected + "&";
 
-	if( selected == "no Script"){
+        if( !QFile( selected ).exists() ){
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Script selected!" ));
 		return;
 	}
@@ -737,10 +741,10 @@ void PlaylistDialog::o_downBClicked()
 void PlaylistDialog::on_m_pPlaylistTree_itemClicked ( QTreeWidgetItem * item, int column )
 {
 	if ( column == 2 ){ 
-		QString selected = "";
+		QString selected;
 		selected = item->text ( 1 );
 
-		if( selected == "no Script"){
+                if( !QFile( selected ).exists() ){
 			QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Script!" ));
 			item->setCheckState( 2, Qt::Unchecked );
 			return;
@@ -759,7 +763,7 @@ void PlaylistDialog::nodePlayBTN( Button* ref )
 	if (ref->isPressed()) {
 		QTreeWidgetItem* m_pPlaylistItem = m_pPlaylistTree->currentItem();
 		if ( m_pPlaylistItem == NULL ){
-			QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Song selected!" ) );
+			QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No song selected!" ) );
 			m_pPlayBtn->setPressed(false);
 			return;
 		}
@@ -832,7 +836,7 @@ void PlaylistDialog::on_m_pPlaylistTree_itemDoubleClicked ()
 		QMessageBox::information ( this, "Hydrogen", trUtf8 ( "No Song selected!" ) );
 		return;
 	}
-	QString selected = "";
+	QString selected;
 	selected = m_pPlaylistItem->text ( 0 );
 
 	QTreeWidget* m_pPlaylist = m_pPlaylistTree;
@@ -850,6 +854,8 @@ void PlaylistDialog::on_m_pPlaylistTree_itemDoubleClicked ()
 
 	m_pPlayBtn->setPressed(false);
 
+	engine->m_timelinetagvector.clear();
+
 	LocalFileMng mng;
 	Song *pSong = Song::load ( selected );
 	if ( pSong == NULL ){
@@ -860,18 +866,20 @@ void PlaylistDialog::on_m_pPlaylistTree_itemDoubleClicked ()
 	pH2App->setSong ( pSong );
 	engine->setSelectedPatternNumber ( 0 );
 
-
-	pH2App->setStatusBarMessage( trUtf8( "Playlist: Set song No. %1" ).arg( index +1 ), 5000 );
+	HydrogenApp::get_instance()->getSongEditorPanel()->updatePositionRuler();
+	pH2App->setStatusBarMessage( trUtf8( "Playlist: set song no. %1" ).arg( index +1 ), 5000 );
 
 	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->update_background_color();
 
+	EventQueue::get_instance()->push_event( EVENT_METRONOME, 3 );
+
 ///exec script
 ///this is very very simple and only an experiment
 #ifdef WIN32
 	//I know nothing about windows scripts -wolke-
 	return;
 #else
-	QString execscript = "";
+	QString execscript;
 	selected = m_pPlaylistItem->text ( 1 );
 	bool execcheckbox = m_pPlaylistItem->checkState ( 2 );
 
@@ -908,7 +916,7 @@ void PlaylistDialog::updatePlayListVector()
 	for (int i = 0 ;i < length; i++){
 		QTreeWidgetItem * m_pPlaylistItem = m_pPlaylist->topLevelItem ( i );	
 		
-		QString execval = "";
+		QString execval;
 		bool execcheckbox = m_pPlaylistItem->checkState ( 2 );
 		if ( execcheckbox == true ) {
 			execval = "Use Script";
@@ -940,14 +948,16 @@ void PlaylistDialog::updateActiveSongNumber()
 	}
 		
 	int selected = Playlist::get_instance()->getActiveSongNumber();
-	if ( selected == -1 ) 
+	if ( selected == -1 )
 		return;
 	
-	QTreeWidgetItem* m_pPlaylistItem = m_pPlaylist->topLevelItem ( selected );	
-	//m_pPlaylist->setCurrentItem ( m_pPlaylistItem );
-	m_pPlaylistItem->setBackgroundColor ( 0, QColor( 50, 50, 50) );
-	m_pPlaylistItem->setBackgroundColor ( 1, QColor( 50, 50, 50) );
-	m_pPlaylistItem->setBackgroundColor ( 2, QColor( 50, 50, 50) );
+	QTreeWidgetItem* m_pPlaylistItem = m_pPlaylist->topLevelItem ( selected );
+	if ( m_pPlaylistItem != NULL ){
+	    //m_pPlaylist->setCurrentItem ( m_pPlaylistItem );
+	    m_pPlaylistItem->setBackgroundColor ( 0, QColor( 50, 50, 50) );
+	    m_pPlaylistItem->setBackgroundColor ( 1, QColor( 50, 50, 50) );
+	    m_pPlaylistItem->setBackgroundColor ( 2, QColor( 50, 50, 50) );
+	}
 }
 
 
@@ -985,3 +995,38 @@ bool PlaylistDialog::eventFilter ( QObject *o, QEvent *e )
 return NULL;
 }
 
+bool PlaylistDialog::loadListByFileName( QString filename )
+{
+
+	LocalFileMng fileMng;
+	int err = fileMng.loadPlayList( filename.toLocal8Bit().constData() );
+	if ( err != 0 ) {
+		_ERRORLOG( "Error loading the playlist" );
+		return 0;
+	}
+	Preferences::get_instance()->setLastPlaylistFilename( filename );
+
+	if(Hydrogen::get_instance()->m_PlayList.size() > 0){
+
+		QTreeWidget* m_pPlaylist = m_pPlaylistTree;
+		m_pPlaylist->clear();
+
+		for ( uint i = 0; i < Hydrogen::get_instance()->m_PlayList.size(); ++i ){
+			QTreeWidgetItem* m_pPlaylistItem = new QTreeWidgetItem ( m_pPlaylistTree );
+			m_pPlaylistItem->setText ( 0, Hydrogen::get_instance()->m_PlayList[i].m_hFile );
+			m_pPlaylistItem->setText ( 1, Hydrogen::get_instance()->m_PlayList[i].m_hScript );
+			if ( Hydrogen::get_instance()->m_PlayList[i].m_hScriptEnabled == "Use Script" ) {
+				m_pPlaylistItem->setCheckState( 2, Qt::Checked );
+			}else{
+				m_pPlaylistItem->setCheckState( 2, Qt::Unchecked );
+			}
+		}
+
+		QTreeWidgetItem* m_pPlaylistItem = m_pPlaylist->topLevelItem ( 0 );
+		m_pPlaylist->setCurrentItem ( m_pPlaylistItem );
+		Playlist::get_instance()->setSelectedSongNr( 0 );
+		Playlist::get_instance()->__playlistName = filename;
+		setWindowTitle ( trUtf8 ( "Playlist Browser" ) + QString(" - ") + QString( Playlist::get_instance()->__playlistName  ) );
+	}
+	return 1;
+}
