@@ -23,6 +23,10 @@
 #include "PortMidiDriver.h"
 
 #include <hydrogen/Preferences.h>
+#include <hydrogen/note.h>
+#include <hydrogen/instrument.h>
+#include <hydrogen/hydrogen.h>
+#include <hydrogen/globals.h>
 
 #ifdef WIN32
 #include <windows.h>
@@ -31,7 +35,7 @@
 #ifdef PORTMIDI_SUPPORT
 
 #include <porttime.h>
-#define TIME_PROC ((PmTimestamp (*)(void *)) Pt_Time)
+#define TIME_PROC ((int32_t (*)(void *)) Pt_Time)
 #define TIME_START Pt_Start(1, 0, 0) /* timer started w/millisecond accuracy */
 
 #include <pthread.h>
@@ -113,7 +117,7 @@ void* PortMidiDriver_thread( void* param )
 
 
 PortMidiDriver::PortMidiDriver()
-		: MidiInput( "PortMidiDriver" )
+		: MidiInput( "PortMidiDriver" ), MidiOutput( "PortMidiDriver" ), Object( "PortMidiDriver" )
 		, m_bRunning( false )
 {
 	Pm_Initialize();
@@ -135,6 +139,7 @@ void PortMidiDriver::open()
 	int nInputBufferSize = 100;
 
 	int nDeviceId = -1;
+	int nOutDeviceId = -1;
 	QString sMidiPortName = Preferences::get_instance()->m_sMidiPortName;
 	int nDevices = Pm_CountDevices();
 	for ( int i = 0; i < nDevices; i++ ) {
@@ -144,16 +149,27 @@ void PortMidiDriver::open()
 		}
 
 		if ( info->input == TRUE ) {
-			if ( info->name == sMidiPortName.toStdString() ) {
+			if ( info->name == sMidiPortName.toLocal8Bit().constData() ) {
 				nDeviceId = i;
 			}
 		}
+		
+		if ( info->output == TRUE ) {
+			if ( info->name == sMidiPortName.toStdString() ) {
+				nOutDeviceId = i;
+			}
+		}
 	}
 
 	if ( nDeviceId == -1 ) {
 		INFOLOG( "Midi input device not found." );
 		return;
 	}
+	
+	if ( nOutDeviceId == -1 ) {
+		INFOLOG( "Midi output device not found." );
+		return;
+	}
 
 	const PmDeviceInfo *info = Pm_GetDeviceInfo( nDeviceId );
 	if ( info == NULL ) {
@@ -175,6 +191,20 @@ void PortMidiDriver::open()
 	if ( err != pmNoError ) {
 		ERRORLOG( "Error in Pm_OpenInput" );
 	}
+	
+	err = Pm_OpenOutput(
+	                  &m_pMidiOut,
+	                  nOutDeviceId,
+	                  NULL,
+	                  nInputBufferSize,
+	                  TIME_PROC,
+	                  NULL,
+			  0
+	              );
+
+	if ( err != pmNoError ) {
+		ERRORLOG( "Error in Pm_OpenInput" );
+	}
 
 	m_bRunning = true;
 
@@ -219,6 +249,83 @@ std::vector<QString> PortMidiDriver::getOutputPortList()
 	return portList;
 }
 
+void PortMidiDriver::handleQueueNote(Note* pNote)
+{	
+	if ( m_pMidiOut == NULL ) {
+		ERRORLOG( "m_pMidiOut = NULL " );
+		return;
+	}
+
+	int channel = pNote->get_instrument()->get_midi_out_channel();
+	if (channel < 0) {
+		return;
+	}
+		
+	int key = (pNote->m_noteKey.m_nOctave +3 ) * 12 + pNote->m_noteKey.m_key + pNote->get_instrument()->get_midi_out_note() -60;
+	int velocity = pNote->get_velocity() * 127;
+	
+	PmEvent event;
+	event.timestamp = 0;
+	
+	//Note off
+	event.message = Pm_Message(0x80 | channel, key, velocity);
+	Pm_Write(m_pMidiOut, &event, 1);
+	
+	//Note on
+	event.message = Pm_Message(0x90 | channel, key, velocity);
+	Pm_Write(m_pMidiOut, &event, 1);
+}
+
+void PortMidiDriver::handleQueueNoteOff( int channel, int key, int velocity )
+{	
+	if ( m_pMidiOut == NULL ) {
+		ERRORLOG( "m_pMidiOut = NULL " );
+		return;
+	}
+
+//	int channel = pNote->get_instrument()->get_midi_out_channel();
+	if (channel < 0) {
+		return;
+	}
+		
+//	int velocity = pNote->get_velocity() * 127;
+	
+	PmEvent event;
+	event.timestamp = 0;
+	
+	//Note off
+	event.message = Pm_Message(0x80 | channel, key, velocity);
+	Pm_Write(m_pMidiOut, &event, 1);
+}
+
+void PortMidiDriver::handleQueueAllNoteOff()
+{
+	if ( m_pMidiOut == NULL ) {
+		ERRORLOG( "m_pMidiOut = NULL " );
+		return;
+	}
+	
+	InstrumentList *instList = Hydrogen::get_instance()->getSong()->get_instrument_list();
+		
+	unsigned int numInstruments = instList->get_size();
+	for (int index = 0; index < numInstruments; ++index) {
+		Instrument *curInst = instList->get(index);
+	
+		int channel = curInst->get_midi_out_channel();
+		if (channel < 0) {
+			continue;
+		}
+		int key = curInst->get_midi_out_note();
+		
+		PmEvent event;
+		event.timestamp = 0;
+	
+		//Note off
+		event.message = Pm_Message(0x80 | channel, key, 0);
+		Pm_Write(m_pMidiOut, &event, 1);
+	}
+}
+
 };
 
 #endif	// PORTMIDI_SUPPORT
