@@ -39,8 +39,18 @@ ActionManager* ActionManager::__instance = NULL;
 using namespace H2Core;
 
 /* Class Action */
-Action::Action( QString s ) : Object( "Action" ) {
-	type = s;
+Action::Action( QString typeString ) : Object( "Action" ) {
+
+	/*
+	    An "Action" is something which can be interpreted and executed by hydrogen.
+	    Example: If hydrogen executes the Action with type "MUTE", it will mute the outputs.
+
+	    The two parameters are optional and can be used to carry additional informations, which mean
+	    only something to this very Action. They can have totally different meanings for other Actions.
+	    Example: parameter1 is the Mixer strip and parameter 2 a multiplier for the volume change on this strip
+	*/
+
+	type = typeString;
 	QString parameter1 = "0";
 	QString parameter2 = "0" ;
 }
@@ -52,9 +62,13 @@ ActionManager::ActionManager() : Object( "ActionManager" )
 {
 	__instance = this;
 	
+	/*
+	    the actionList holds all Action identfiers which hydrogen is able to interpret.
+	*/
 	actionList <<""
 	<< "PLAY" 
-	<< "PLAY_TOGGLE"
+	<< "PLAY/STOP_TOGGLE"
+	<< "PLAY/PAUSE_TOGGLE"
 	<< "STOP"
 	<< "PAUSE"
 	<< "MUTE"
@@ -78,10 +92,12 @@ ActionManager::ActionManager() : Object( "ActionManager" )
 	<< "EFFECT3_LEVEL_ABSOLUTE"
 	<< "EFFECT4_LEVEL_ABSOLUTE"
 	<< "SELECT_NEXT_PATTERN"
+        << "SELECT_AND_PLAY_NEXT_PATTERN"
 	<< "PAN_RELATIVE"
-	<< "PAN_ABSOULTE"
+	<< "PAN_ABSOLUTE"
 	<< "BEATCOUNTER"
-	<< "TAP_TEMPO";
+	<< "TAP_TEMPO"
+	<< "SELECT_INSTRUMENT";
 
 	eventList << ""
 	<< "MMC_PLAY"
@@ -157,7 +173,7 @@ bool ActionManager::handleAction( Action * pAction ){
 		return true;
 	}
 
-	if( sActionString == "PLAY_TOGGLE" )
+	if( sActionString == "PLAY/STOP_TOGGLE" || sActionString == "PLAY/PAUSE_TOGGLE" )
 	{
 		int nState = pEngine->getState();
 		switch ( nState ) 
@@ -167,7 +183,9 @@ bool ActionManager::handleAction( Action * pAction ){
 				break;
 
 			case STATE_PLAYING:
+				if( sActionString == "PLAY/STOP_TOGGLE" ) pEngine->setPatternPos( 0 );
 				pEngine->sequencer_stop();
+				pEngine->setTimelineBpm();
 				break;
 
 			default:
@@ -176,6 +194,7 @@ bool ActionManager::handleAction( Action * pAction ){
 
 		return true;
 	}
+	
 
 	if( sActionString == "PAUSE" )
 	{	
@@ -187,6 +206,7 @@ bool ActionManager::handleAction( Action * pAction ){
 	{	
 		pEngine->sequencer_stop();
 		pEngine->setPatternPos( 0 );
+		pEngine->setTimelineBpm();
 		return true;
 	}
 
@@ -216,6 +236,38 @@ bool ActionManager::handleAction( Action * pAction ){
 		return true;
 	}
 
+	  if( sActionString == "SELECT_NEXT_PATTERN"){
+		bool ok;
+		int row = pAction->getParameter1().toInt(&ok,10);
+		pEngine->setSelectedPatternNumber( row );
+		pEngine->sequencer_setNextPattern( row, false, true );
+                return true;
+        }
+
+          if( sActionString == "SELECT_AND_PLAY_NEXT_PATTERN"){
+                bool ok;
+                int row = pAction->getParameter1().toInt(&ok,10);
+                pEngine->setSelectedPatternNumber( row );
+                pEngine->sequencer_setNextPattern( row, false, true );
+
+                int nState = pEngine->getState();
+                if ( nState == STATE_READY ){
+                        pEngine->sequencer_play();
+                }
+
+                return true;
+        }
+
+
+	if( sActionString == "SELECT_INSTRUMENT" ){
+		bool ok;
+		int  instrument_number = pAction->getParameter2().toInt(&ok,10) ;
+		if ( pEngine->getSong()->get_instrument_list()->get_size() < instrument_number ) 
+			instrument_number = pEngine->getSong()->get_instrument_list()->get_size() -1;	
+		pEngine->setSelectedInstrumentNumber( instrument_number );
+		return true;
+	}
+
 
 
 	
@@ -357,17 +409,8 @@ bool ActionManager::handleAction( Action * pAction ){
 		Hydrogen::get_instance()->setSelectedInstrumentNumber(nLine);
 	}
 
-	if( sActionString == "SELECT_NEXT_PATTERN"){
-		bool ok;
-		int row = pAction->getParameter1().toInt(&ok,10);
-		pEngine->setSelectedPatternNumber( row );
-		pEngine->sequencer_setNextPattern( row, false, true );
-       		return true;
-	}
-
-
-
-	if( sActionString == "PAN_ABSOULTE" ){
+	
+	if( sActionString == "PAN_ABSOLUTE" ){
 		
 		 // sets the absolute panning of a given mixer channel
 		
@@ -568,11 +611,13 @@ bool ActionManager::handleAction( Action * pAction ){
 
 	if( sActionString == ">>_NEXT_BAR"){
 		pEngine->setPatternPos(pEngine->getPatternPos() +1 );
+		pEngine->setTimelineBpm();
 		return true;
 	}
 
 	if( sActionString == "<<_PREVIOUS_BAR"){
 		pEngine->setPatternPos(pEngine->getPatternPos() -1 );
+		pEngine->setTimelineBpm();
 		return true;
 	}
 	
