@@ -36,10 +36,18 @@
 #include <hydrogen/Preferences.h>
 #include <hydrogen/sample.h>
 #include <hydrogen/Song.h>
+#include <hydrogen/Pattern.h>
+#include <hydrogen/event_queue.h>
+
+#include "gui/src/HydrogenApp.h"
+#include "gui/src/PatternEditor/PatternEditorPanel.h"
+#include "gui/src/PatternEditor/DrumPatternEditor.h"
 
 #include <hydrogen/fx/Effects.h>
 #include <hydrogen/sampler/Sampler.h>
 
+#include <iostream>
+
 namespace H2Core
 {
 
@@ -50,8 +58,6 @@ inline static float linear_interpolation( float fVal_A, float fVal_B, float fVal
 //	return fVal_A + ((fVal_B - fVal_A) * fVal);
 }
 
-
-
 Sampler::Sampler()
 		: Object( "Sampler" )
 		, __main_out_L( NULL )
@@ -68,6 +74,7 @@ Sampler::Sampler()
 	__preview_instrument = new Instrument( sEmptySampleFilename, "preview", new ADSR() );
 	__preview_instrument->set_volume( 0.8 );
 	__preview_instrument->set_layer( new InstrumentLayer( Sample::load( sEmptySampleFilename ) ), 0 );
+
 }
 
 
@@ -93,8 +100,8 @@ void Sampler::process( uint32_t nFrames, Song* pSong )
 	memset( __main_out_L, 0, nFrames * sizeof( float ) );
 	memset( __main_out_R, 0, nFrames * sizeof( float ) );
 
-	// Track output queues are zeroed by
-	// audioEngine_process_clearAudioBuffers()
+	// Track output queues are zeroed by 
+ 	// audioEngine_process_clearAudioBuffers() 
 
 	// Max notes limit
 	int m_nMaxNotes = Preferences::get_instance()->m_nMaxNotes;
@@ -115,12 +122,31 @@ void Sampler::process( uint32_t nFrames, Song* pSong )
 		if ( res == 1 ) {	// la nota e' finita
 			__playing_notes_queue.erase( __playing_notes_queue.begin() + i );
 			pNote->get_instrument()->dequeue();
-			delete pNote;
-			pNote = NULL;
+			__queuedNoteOffs.push_back( pNote );
+//			delete pNote;
+//			pNote = NULL;
 		} else {
 			++i; // carico la prox nota
 		}
 	}
+	
+	//Queue midi note off messages for notes that have a length specified for them
+
+	while ( !__queuedNoteOffs.empty() ) {
+		pNote =  __queuedNoteOffs[0];
+		MidiOutput* midiOut = Hydrogen::get_instance()->getMidiOutput();
+		if( midiOut != NULL ){ 
+		    midiOut->handleQueueNoteOff( pNote->get_instrument()->get_midi_out_channel(),
+					        ( pNote->m_noteKey.m_nOctave +3 ) * 12 + pNote->m_noteKey.m_key +
+					        ( pNote->get_instrument()->get_midi_out_note() -60 ), 
+					        pNote->get_velocity() * 127 );
+
+		}
+		__queuedNoteOffs.erase( __queuedNoteOffs.begin() );
+		if( pNote != NULL) delete pNote;
+		pNote = NULL;
+	}//while
+
 }
 
 
@@ -130,41 +156,75 @@ void Sampler::note_on( Note *note )
 	//infoLog( "[noteOn]" );
 	assert( note );
 
-	// mute groups
 	Instrument *pInstr = note->get_instrument();
+
+	// mute groups	
 	if ( pInstr->get_mute_group() != -1 ) {
 		// remove all notes using the same mute group
 		for ( unsigned j = 0; j < __playing_notes_queue.size(); j++ ) {	// delete older note
 			Note *pNote = __playing_notes_queue[ j ];
-
 			if ( ( pNote->get_instrument() != pInstr )  && ( pNote->get_instrument()->get_mute_group() == pInstr->get_mute_group() ) ) {
-				//warningLog("release");
 				pNote->m_adsr.release();
 			}
 		}
 	}
+
+	//note off notes	
+	if( note->get_noteoff() ){
+		for ( unsigned j = 0; j < __playing_notes_queue.size(); j++ ) {
+			Note *pNote = __playing_notes_queue[ j ];
+
+			if ( ( pNote->get_instrument() == pInstr ) ) {
+				//ERRORLOG("note_off");
+				pNote->m_adsr.release();
+			}	
+		}
+	}
 	
 	pInstr->enqueue();
-	__playing_notes_queue.push_back( note );
+	if( !note->get_noteoff() ){
+		__playing_notes_queue.push_back( note );
+	} else {
+		delete note;
+	}
+	
+	if( Hydrogen::get_instance()->getMidiOutput() != NULL ){
+		Hydrogen::get_instance()->getMidiOutput()->handleQueueNote( note );
+	}
+	
+}
+
+void Sampler::midi_keyboard_note_off( int key )
+{
+	for ( unsigned j = 0; j < __playing_notes_queue.size(); j++ ) {
+		Note *pNote = __playing_notes_queue[ j ];
+
+		if ( ( pNote->get_midimsg1() == key) ) {
+			pNote->m_adsr.release();
+		}	
+	}
 }
 
 
 
 void Sampler::note_off( Note* note )
+	/*
+	* this old note_off function is only used by right click on mixer channel strip play button
+	* all other note_off stuff will handle in midi_keyboard_note_off() and note_on()
+	*/
 {
-	assert(note);
+
 	Instrument *pInstr = note->get_instrument();
 	// find the notes using the same instrument, and release them
 	for ( unsigned j = 0; j < __playing_notes_queue.size(); j++ ) {
-		Note *pNote = __playing_notes_queue[ j ];
+ 		Note *pNote = __playing_notes_queue[ j ];
 		if ( pNote->get_instrument() == pInstr ) {
 			pNote->m_adsr.release();
 		}
-	}
+ 	}
 }
 
 
-
 /// Render a note
 /// Return 0: the note is not ended
 /// Return 1: the note is ended
@@ -183,7 +243,6 @@ unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong
 		nFramepos = pEngine->getRealtimeFrames();
 	}
 
-
 	Instrument *pInstr = pNote->get_instrument();
 	if ( !pInstr ) {
 		ERRORLOG( "NULL instrument" );
@@ -240,9 +299,7 @@ unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong
 	float cost_L = 1.0f;
 	float cost_R = 1.0f;
 	float cost_track_L = 1.0f;
-	float cost_track_R = 1.0f;
-	float fSendFXLevel_L = 1.0f;
-	float fSendFXLevel_R = 1.0f;
+        float cost_track_R = 1.0f;
 
 	if ( pInstr->is_muted() || pSong->__is_muted ) {	// is instrument muted?
 		cost_L = 0.0;
@@ -253,21 +310,19 @@ unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong
 			cost_track_R = 0.0;
 		}
 
-		fSendFXLevel_L = 0.0f;
-		fSendFXLevel_R = 0.0f;
 	} else {	// Precompute some values...
 		cost_L = cost_L * pNote->get_velocity();		// note velocity
 		cost_L = cost_L * pNote->get_pan_l();		// note pan
 		cost_L = cost_L * fLayerGain;				// layer gain
 		cost_L = cost_L * pInstr->get_pan_l();		// instrument pan
-		cost_L = cost_L * pInstr->get_gain();		// instrument gain
-		fSendFXLevel_L = cost_L;
+                cost_L = cost_L * pInstr->get_gain();		// instrument gain
 
 		cost_L = cost_L * pInstr->get_volume();		// instrument volume
                 if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
 		// Post-Fader
 			cost_track_L = cost_L * 2;
 		}
+
 		cost_L = cost_L * pSong->get_volume();	// song volume
 		cost_L = cost_L * 2; // max pan is 0.5
 
@@ -276,8 +331,7 @@ unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong
 		cost_R = cost_R * pNote->get_pan_r();		// note pan
 		cost_R = cost_R * fLayerGain;				// layer gain
 		cost_R = cost_R * pInstr->get_pan_r();		// instrument pan
-		cost_R = cost_R * pInstr->get_gain();		// instrument gain
-		fSendFXLevel_R = cost_R;
+                cost_R = cost_R * pInstr->get_gain();		// instrument gain
 
 		cost_R = cost_R * pInstr->get_volume();		// instrument volume
                 if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
@@ -307,9 +361,9 @@ unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong
 	//_INFOLOG( "total pitch: " + to_string( fTotalPitch ) );
 
 	if ( fTotalPitch == 0.0 && pSample->get_sample_rate() == audio_output->getSampleRate() ) {	// NO RESAMPLE
-		return __render_note_no_resample( pSample, pNote, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, fSendFXLevel_L, fSendFXLevel_R, pSong );
+                return __render_note_no_resample( pSample, pNote, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, pSong );
 	} else {	// RESAMPLE
-		return __render_note_resample( pSample, pNote, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, fLayerPitch, fSendFXLevel_L, fSendFXLevel_R, pSong );
+                return __render_note_resample( pSample, pNote, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, fLayerPitch, pSong );
 	}
 }
 
@@ -325,8 +379,6 @@ int Sampler::__render_note_no_resample(
     float cost_R,
     float cost_track_L,
     float cost_track_R,
-    float fSendFXLevel_L,
-    float fSendFXLevel_R,
     Song* pSong
 )
 {
@@ -346,6 +398,7 @@ int Sampler::__render_note_no_resample(
 		retValue = 0; // the note is not ended yet
 	}
 
+
 	//ADSR *pADSR = pNote->m_pADSR;
 
 	int nInitialBufferPos = nInitialSilence;
@@ -378,16 +431,17 @@ int Sampler::__render_note_no_resample(
 		nInstrument = 0;
 	}
 
-#ifdef JACK_SUPPORT
-	JackOutput* jao = 0;
-	float *track_out_L = 0;
-	float *track_out_R = 0;
-	if( audio_output->has_track_outs()
-	    && (jao = dynamic_cast<JackOutput*>(audio_output)) ) {
-		track_out_L = jao->getTrackOut_L( nInstrument );
-		track_out_R = jao->getTrackOut_R( nInstrument );
-	}
-#endif
+#ifdef JACK_SUPPORT 
+	JackOutput* jao = 0; 
+	float *track_out_L = 0; 
+	float *track_out_R = 0; 
+	if( audio_output->has_track_outs() 
+	&& (jao = dynamic_cast<JackOutput*>(audio_output)) ) { 
+		track_out_L = jao->getTrackOut_L( nInstrument ); 
+		track_out_R = jao->getTrackOut_R( nInstrument ); 
+	} 
+#endif 
+	
 	for ( int nBufferPos = nInitialBufferPos; nBufferPos < nTimes; ++nBufferPos ) {
 		if ( ( nNoteLength != -1 ) && ( nNoteLength <= pNote->m_fSamplePosition )  ) {
 			if ( pNote->m_adsr.release() == 0 ) {
@@ -411,12 +465,12 @@ int Sampler::__render_note_no_resample(
 		}
 
 #ifdef JACK_SUPPORT
-		if( track_out_L ) {
-			track_out_L[nBufferPos] += fVal_L * cost_track_L;
-		}
-		if( track_out_R ) {
-			track_out_R[nBufferPos] += fVal_R * cost_track_R;
-		}
+	if( track_out_L ) { 
+		track_out_L[nBufferPos] += fVal_L * cost_track_L; 
+	} 
+	if( track_out_R ) { 
+		track_out_R[nBufferPos] += fVal_R * cost_track_R; 
+	}
 #endif
 
                 fVal_L = fVal_L * cost_L;
@@ -440,9 +494,10 @@ int Sampler::__render_note_no_resample(
 	pNote->get_instrument()->set_peak_l( fInstrPeak_L );
 	pNote->get_instrument()->set_peak_r( fInstrPeak_R );
 
-
 #ifdef LADSPA_SUPPORT
 	// LADSPA
+
+        float masterVol = pSong->get_volume();
 	for ( unsigned nFX = 0; nFX < MAX_FX; ++nFX ) {
 		LadspaFX *pFX = Effects::get_instance()->getLadspaFX( nFX );
 
@@ -455,14 +510,14 @@ int Sampler::__render_note_no_resample(
 
 //			float fFXCost_L = cost_L * fLevel;
 //			float fFXCost_R = cost_R * fLevel;
-			float fFXCost_L = fLevel * fSendFXLevel_L;
-			float fFXCost_R = fLevel * fSendFXLevel_R;
+                        float fFXCost_L = fLevel * masterVol;
+                        float fFXCost_R = fLevel * masterVol;
 
 			int nBufferPos = nInitialBufferPos;
 			int nSamplePos = nInitialSamplePos;
 			for ( int i = 0; i < nAvail_bytes; ++i ) {
-				pBuf_L[ nBufferPos ] += pSample_data_L[ nSamplePos ] * fFXCost_L * cost_L;
-				pBuf_R[ nBufferPos ] += pSample_data_R[ nSamplePos ] * fFXCost_R * cost_R;
+                                pBuf_L[ nBufferPos ] += pSample_data_L[ nSamplePos ] * fFXCost_L;
+                                pBuf_R[ nBufferPos ] += pSample_data_R[ nSamplePos ] * fFXCost_R;
 				++nSamplePos;
 				++nBufferPos;
 			}
@@ -486,8 +541,6 @@ int Sampler::__render_note_resample(
     float cost_track_L,
     float cost_track_R,
     float fLayerPitch,
-    float fSendFXLevel_L,
-    float fSendFXLevel_R,
     Song* pSong
 )
 {
@@ -499,9 +552,8 @@ int Sampler::__render_note_resample(
 	float fNotePitch = pNote->get_pitch() + fLayerPitch;
 	fNotePitch += pNote->m_noteKey.m_nOctave * 12 + pNote->m_noteKey.m_key;
 
-	//_INFOLOG( "pitch: " + to_string( fNotePitch ) );
-
 	float fStep = pow( 1.0594630943593, ( double )fNotePitch );
+//	_ERRORLOG( QString("pitch: %1, step: %2" ).arg(fNotePitch).arg( fStep) );
 	fStep *= ( float )pSample->get_sample_rate() / audio_output->getSampleRate(); // Adjust for audio driver sample rate
 
 	int nAvail_bytes = ( int )( ( float )( pSample->get_n_frames() - pNote->m_fSamplePosition ) / fStep );	// verifico il numero di frame disponibili ancora da eseguire
@@ -546,17 +598,17 @@ int Sampler::__render_note_resample(
 		nInstrument = 0;
 	}
 
-
-#ifdef JACK_SUPPORT
-	JackOutput* jao = 0;
-	float *track_out_L = 0;
-	float *track_out_R = 0;
-	if( audio_output->has_track_outs()
-	    && (jao = dynamic_cast<JackOutput*>(audio_output)) ) {
-		track_out_L = jao->getTrackOut_L( nInstrument );
-		track_out_R = jao->getTrackOut_R( nInstrument );
-	}
+#ifdef JACK_SUPPORT 
+	JackOutput* jao = 0; 
+	float *track_out_L = 0; 
+	float *track_out_R = 0; 
+	if( audio_output->has_track_outs() 
+	&& (jao = dynamic_cast<JackOutput*>(audio_output)) ) { 
+		track_out_L = jao->getTrackOut_L( nInstrument ); 
+		track_out_R = jao->getTrackOut_R( nInstrument ); 
+	} 
 #endif
+
 	for ( int nBufferPos = nInitialBufferPos; nBufferPos < nTimes; ++nBufferPos ) {
 		if ( ( nNoteLength != -1 ) && ( nNoteLength <= pNote->m_fSamplePosition )  ) {
 			if ( pNote->m_adsr.release() == 0 ) {
@@ -564,14 +616,16 @@ int Sampler::__render_note_resample(
 			}
 		}
 
-		int nSamplePos = ( int )fSamplePos;
-		double fDiff = fSamplePos - nSamplePos;
-		if ( ( nSamplePos + 1 ) >= nSampleFrames ) {
-			fVal_L = linear_interpolation( pSample_data_L[ nSampleFrames ], 0, fDiff );
-			fVal_R = linear_interpolation( pSample_data_R[ nSampleFrames ], 0, fDiff );
+                int nSamplePos = ( int )fSamplePos;
+                double fDiff = fSamplePos - nSamplePos;
+                if ( ( nSamplePos + 1 ) >= nSampleFrames ) {
+                        //we reach the last audioframe.
+			//set this last frame to zero do nothin wrong.
+			fVal_L = 0.0;
+			fVal_R = 0.0;
 		} else {
-			fVal_L = linear_interpolation( pSample_data_L[nSamplePos], pSample_data_L[nSamplePos + 1], fDiff );
-			fVal_R = linear_interpolation( pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], fDiff );
+                        fVal_L = linear_interpolation( pSample_data_L[nSamplePos], pSample_data_L[nSamplePos + 1], fDiff );
+                        fVal_R = linear_interpolation( pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], fDiff );
 		}
 
 		// ADSR envelope
@@ -592,12 +646,12 @@ int Sampler::__render_note_resample(
 
 
 #ifdef JACK_SUPPORT
-		if( track_out_L ) {
-			track_out_L[nBufferPos] += (fVal_L * cost_track_L);
-		}
-		if( track_out_R ) {
-			track_out_R[nBufferPos] += (fVal_R * cost_track_R);
-		}
+	if( track_out_L ) { 
+		track_out_L[nBufferPos] += fVal_L * cost_track_L; 
+	} 
+	if( track_out_R ) { 
+		track_out_R[nBufferPos] += fVal_R * cost_track_R; 
+	}
 #endif
 
 		fVal_L = fVal_L * cost_L;
@@ -625,6 +679,7 @@ int Sampler::__render_note_resample(
 
 #ifdef LADSPA_SUPPORT
 	// LADSPA
+        float masterVol = pSong->get_volume();
 	for ( unsigned nFX = 0; nFX < MAX_FX; ++nFX ) {
 		LadspaFX *pFX = Effects::get_instance()->getLadspaFX( nFX );
 		float fLevel = pNote->get_instrument()->get_fx_level( nFX );
@@ -636,8 +691,8 @@ int Sampler::__render_note_resample(
 
 //			float fFXCost_L = cost_L * fLevel;
 //			float fFXCost_R = cost_R * fLevel;
-			float fFXCost_L = fLevel * fSendFXLevel_L;
-			float fFXCost_R = fLevel * fSendFXLevel_R;
+                        float fFXCost_L = fLevel * masterVol;
+                        float fFXCost_R = fLevel * masterVol;
 
 			int nBufferPos = nInitialBufferPos;
 			float fSamplePos = fInitialSamplePos;
@@ -645,16 +700,18 @@ int Sampler::__render_note_resample(
 				int nSamplePos = ( int )fSamplePos;
 				double fDiff = fSamplePos - nSamplePos;
 
-				if ( ( nSamplePos + 1 ) >= nSampleFrames ) {
-					fVal_L = linear_interpolation( pSample_data_L[nSamplePos], 0, fDiff );
-					fVal_R = linear_interpolation( pSample_data_R[nSamplePos], 0, fDiff );
+                                if ( ( nSamplePos + 1 ) >= nSampleFrames ) {
+                                        //we reach the last audioframe.
+					//set this last frame to zero do nothin wrong.
+					fVal_L = 0.0;
+					fVal_R = 0.0;
 				} else {
 					fVal_L = linear_interpolation( pSample_data_L[nSamplePos], pSample_data_L[nSamplePos + 1], fDiff );
-					fVal_R = linear_interpolation( pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], fDiff );
+                                        fVal_R = linear_interpolation( pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], fDiff );
 				}
 
-				pBuf_L[ nBufferPos ] += fVal_L * fFXCost_L * cost_L;
-				pBuf_R[ nBufferPos ] += fVal_R * fFXCost_R * cost_R;
+                                pBuf_L[ nBufferPos ] += fVal_L * fFXCost_L;
+                                pBuf_R[ nBufferPos ] += fVal_R * fFXCost_R;
 				fSamplePos += fStep;
 				++nBufferPos;
 			}
@@ -668,13 +725,6 @@ int Sampler::__render_note_resample(
 
 void Sampler::stop_playing_notes( Instrument* instrument )
 {
-	/*
-	// send a note-off event to all notes present in the playing note queue
-	for ( int i = 0; i < __playing_notes_queue.size(); ++i ) {
-		Note *pNote = __playing_notes_queue[ i ];
-		pNote->m_pADSR->release();
-	}
-	*/
 
 	if ( instrument ) { // stop all notes using this instrument
 		for ( unsigned i = 0; i < __playing_notes_queue.size(); ) {
@@ -739,5 +789,89 @@ void Sampler::preview_instrument( Instrument* instr )
 }
 
 
-};
 
+void Sampler::setPlayingNotelength( Instrument* instrument, unsigned long ticks, unsigned long noteOnTick )
+{
+	if ( instrument ) { // stop all notes using this instrument
+		Hydrogen *pEngine = Hydrogen::get_instance();	
+		Song* mSong = pEngine->getSong();
+		int selectedpattern = pEngine->__get_selected_PatterNumber();
+		Pattern* currentPattern = NULL;
+
+
+		if ( mSong->get_mode() == Song::PATTERN_MODE ||
+		( pEngine->getState() != STATE_PLAYING )){
+			PatternList *pPatternList = mSong->get_pattern_list();
+			if ( ( selectedpattern != -1 )
+			&& ( selectedpattern < ( int )pPatternList->get_size() ) ) {
+				currentPattern = pPatternList->get( selectedpattern );
+			}
+		}else
+		{
+			std::vector<PatternList*> *pColumns = mSong->get_pattern_group_vector();
+//			Pattern *pPattern = NULL;
+			int pos = pEngine->getPatternPos() +1;
+			for ( int i = 0; i < pos; ++i ) {
+				PatternList *pColumn = ( *pColumns )[i];
+				currentPattern = pColumn->get( 0 );	
+			}
+		}
+
+		
+		if ( currentPattern ) {
+				int patternsize = currentPattern->get_length();
+	
+				for ( unsigned nNote = 0 ;
+				nNote < currentPattern->get_length() ;
+				nNote++ ) {
+					std::multimap <int, Note*>::iterator pos;
+					for ( pos = currentPattern->note_map.lower_bound( nNote ) ;
+					pos != currentPattern->note_map.upper_bound( nNote ) ;
+					++pos ) {
+						Note *pNote = pos->second;
+						if ( pNote!=NULL ) {
+							if( !Preferences::get_instance()->__playselectedinstrument ){
+								if ( pNote->get_instrument() == instrument
+								&& pNote->get_position() == noteOnTick ) {
+									AudioEngine::get_instance()->lock( RIGHT_HERE );
+					
+									if ( ticks >  patternsize ) 
+										ticks = patternsize - noteOnTick;
+									pNote->set_length( ticks );
+									Hydrogen::get_instance()->getSong()->__is_modified = true;
+									AudioEngine::get_instance()->unlock(); // unlock the audio engine
+								}
+							}else
+							{
+								if ( pNote->get_instrument() == pEngine->getSong()->get_instrument_list()->get( pEngine->getSelectedInstrumentNumber())
+								&& pNote->get_position() == noteOnTick ) {
+									AudioEngine::get_instance()->lock( RIGHT_HERE );
+									if ( ticks >  patternsize ) 
+										ticks = patternsize - noteOnTick;
+									pNote->set_length( ticks );
+									Hydrogen::get_instance()->getSong()->__is_modified = true;
+									AudioEngine::get_instance()->unlock(); // unlock the audio engine	
+								}	
+							}
+						}
+					}
+				}
+			}	
+		}
+	EventQueue::get_instance()->push_event( EVENT_PATTERN_MODIFIED, -1 );
+}
+
+bool Sampler::is_instrument_playing( Instrument* instrument )
+{
+
+	if ( instrument ) { // stop all notes using this instrument
+		for ( unsigned j = 0; j < __playing_notes_queue.size(); j++ ) {
+			if ( instrument->get_name() == __playing_notes_queue[ j ]->get_instrument()->get_name()){
+				return true;
+			}
+		}
+		
+	}
+}
+
+};
